/*
******************************************************************************
*  Name     : FastReadStream.h
*  Date     : 3.8.04
*  Description:
*     A Class for fast file access.
*	  Just for internal use.
*     
*  Copyright: Susi Games, Germany
******************************************************************************
*/

#ifndef __FASTREADSTREAM_H__
#define __FASTREADSTREAM_H__

#include <e32base.h>
#include <f32file.h>
#include <s32file.h>
#include "BitmapUtilsDefinitions.h"


#define FASTSTREAM_BUFFERSIZE 3584


/**
 * class CFastReadStream: A Class for fast file access.
 *
 * @author SusiGames.com
 */
class CFastReadStream : public CBase
{
public:

    static CFastReadStream* NewL();
    static CFastReadStream* NewLC();


	/**
	 * Open: Opens a file.
	 *
	 * @param aFilename 
	 * @param aInitBuffer If EFalse, you have to call UpdateBuffer before accessing the data
	 * @return bool 
	 */
	bool Open(TDesC* aFilename, TBool aInitBuffer = ETrue);


	/**
	 * GetFile: To get the standard file handle.
	 *
	 * @return RFile* 
	 */
	RFile* GetFile();


	/**
	 * Close: To close the file stream.
	 *
	 * @return void 
	 */
	void Close();


    /**
     * ~CFastReadStream: Destructor.
     *
     * @return  
     */
    ~CFastReadStream();


	/**
	 * GetUint16: To get an TUint16 from the stream.
	 *
	 * @return inline TUint16 
	 */
	inline TUint16 GetUint16()
	{
		CheckBuffer(2);
		TUint16 res = *((TUint16*)iBufferPtr);
		iBufferPtr+=2;
		iPosInBuffer+=2;
		return res;
	}


	/**
	 * ReadShortChar: Read a short(16bit) and a char(8bit) from the stream.
	 *
	 * @param aShort 
	 * @param aChar 
	 * @return inline void 
	 */
	inline void ReadShortChar(TUint16 &aShort, TUint8 &aChar)
	{
		CheckBuffer(3);
		aShort = *((TUint16*)iBufferPtr);
		iBufferPtr+=2;
		aChar = *((TUint8*)iBufferPtr);
		iBufferPtr++;
		iPosInBuffer+=3;
	}


	/**
	 * Read2Char: Get two chars from the stream.
	 *
	 * @param aChar1 
	 * @param aChar2 
	 * @return inline void 
	 */
	inline void Read2Char(TUint8 &aChar1, TUint8 &aChar2)
	{
		CheckBuffer(2);
		aChar1 = *((TUint8*)iBufferPtr);
		iBufferPtr++;
		aChar2 = *((TUint8*)iBufferPtr);
		iBufferPtr++;
		iPosInBuffer+=2;
	}


	/**
	 * CheckBuffer: Checks if we can read aMinDstToEnd bytes from the buffer.
	 *
	 * @param aMinDstToEnd 
	 * @return inline void 
	 */
	inline void CheckBuffer(TInt aMinDstToEnd)
	{

		if (iPosInBuffer>=(TUint)(iBuffer.Size()-aMinDstToEnd))
		{
			TInt dist = iPosInBuffer-iBuffer.Size();
			if (dist!=0)
			{
				// go back 
				iFile.Seek(ESeekCurrent,dist);
			}
			iBufferPosInFile+=iBuffer.Size()-dist;
			UpdateBuffer();
		}

	}


	/**
	 * StreamOK: Is Stream in a correct state.
	 *
	 * @return inline TBool 
	 */
	inline TBool StreamOK()
	{
		return (iBufferPtr != NULL);
	}



	/**
	 * UpdateBuffer: Load data from file.
	 *
	 * @return inline void 
	 */
	inline void UpdateBuffer()
	{
		iPosInBuffer = 0;
		TInt err;
		TRAP(err,iFile.Read(iBuffer))
		if (err!=KErrNone || iBuffer.Length()==0)
		{
			Close();
			return;
		}
		iBufferPtr = iBuffer.Ptr();
	}


	/**
	 * LengthToEnd: The length to end of file in bytes.
	 *
	 * @return inline TUint 
	 */
	inline TUint LengthToEnd()
	{
		return (iFileSize - iBufferPosInFile - iPosInBuffer);
	}

	
	/**
	 * SeekFromBegin: Go to position aPosition in Stream.
	 *
	 * @param aPosition 
	 * @param aInitBuffer 
	 * @return inline void 
	 */
	inline void SeekFromBegin(TInt aPosition, TBool aInitBuffer = ETrue)
	{
		iFile.Seek(ESeekStart,aPosition);
		iBufferPosInFile = aPosition;
		if (aInitBuffer)
		{
			UpdateBuffer();
		}
	}


	/**
	 * SeekCurrentPosition: Go to relative position aLength in stream.
	 *
	 * @param aLength 
	 * @param aInitBuffer 
	 * @return inline void 
	 */
	inline void SeekCurrentPosition(TInt aLength, TBool aInitBuffer = ETrue)
	{
		iFile.Seek(ESeekCurrent,aLength); // writes actual poistion to aLength
		iBufferPosInFile = aLength;
		if (aInitBuffer)
		{
			UpdateBuffer();
		}
	}

	
	// !! reads only 16 bit data, with 32bit we get some errors on the device?! !!!!!!!!!!!!!!!!!
	// to access files fast:no checks!
	// call this only if the file is open
	inline TUint32 GetUint32()
	{
		CheckBuffer(4);

#ifdef __WINS__
		TUint32 res = *((TUint32*)iBufferPtr);
#else
		TUint32 res = (TUint32)(*((TUint16*)iBufferPtr));
		//TUint32 *readTmp32P = (TUint32*)(&(iBufferPtr[0]));
		//TUint32 res = *(readTmp32P);

#endif
		iBufferPtr+=4;
		iPosInBuffer+=4;
		return res;
	}

private:

    CFastReadStream();
    void ConstructL();

	RFs iFileServerSession;
	RFile iFile;
		
	TBuf8<FASTSTREAM_BUFFERSIZE> iBuffer;
	const TUint8* iBufferPtr;

	TUint iBufferPosInFile;
	TUint iPosInBuffer;

	TInt iFileSize;


};

#endif // __FASTREADSTREAM_H__

